/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla XForms support.
 *
 * The Initial Developer of the Original Code is
 * Olli Pettay.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Olli Pettay <Olli.Pettay@helsinki.fi> (original author)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "nsISupports.idl"
#include "nsIDOMNode.idl"
#include "nsIXFormsDelegate.idl"

/**
 * Interface implemented by the item element.
 */
[scriptable, uuid(ec8d3556-8ed2-4143-88d1-6b7b2c8b0b3b)]
interface nsIXFormsItemElement : nsISupports
{
  /**
   * The text value of the \<label\> child element of the item.
   */
  readonly attribute AString labelText;

  /**
   * The value of the item element.
   */
  readonly attribute AString value;

  /**
   * Marks item active. In current implementation '_moz_active' attribute is 
   * set to the element if aActive is true. The attribute can be used when
   * styling the element.
   */
  void setActive(in boolean aActive);

  /**
   * This is called by the \<label\> child element whenever it is refreshed.
   * This information will be propagated by the \<item\> to the nearest
   * \<select1\> element, which can then refresh its UI.
   */
  void labelRefreshed();

  /**
   * Indicates whether the item element contains a value child or a copy
   * child.  We'll assume that if the item is NOT a copy item, then it must
   * be a value item.  Which means that it must contain a XForms value element
   * child.
   */
  attribute boolean isCopyItem;

  /*
   * returns the node that the contained copy element is bound to
   */
  readonly attribute nsIDOMNode copyNode;

};
