/*
 *  This defines the look-and-feel styling of the error pages.
 *  (see: netError.xhtml)
 *
 *  Original styling by William Price <bugzilla@mob.rice.edu>
 *  Updated by: Steven Garrity <steven@silverorange.com>
 *              Henrik Skupin  <mozilla@hskupin.info>
 */

html {
  background: -moz-Dialog;
}

body {
  margin: 0;
  padding: 0 1em;
  color: -moz-FieldText;
  font: message-box;
}

h1 {
  margin: 0 0 .6em 0;
  border-bottom: 1px solid ThreeDLightShadow;
  font-size: 160%;
}

ul, ol {
  margin: 0;
  -moz-margin-start: 1.5em;
  padding: 0;
}

ul > li, ol > li {
  margin-bottom: .5em;
}

ul {
  list-style: square;
}

#errorPageContainer {
  position: relative;
  min-width: 13em;
  max-width: 52em;
  margin: 4em auto;
  border: 1px solid ThreeDShadow;
  -moz-border-radius: 10px;
  padding: 3em;
  -moz-padding-start: 30px;
  background: url("chrome://global/skin/icons/warning-large.png") left 0 no-repeat -moz-Field;
  -moz-background-origin: content;
}

body[dir="rtl"] #errorPageContainer {
  background-position: right 0;
}

#errorTitle {
  -moz-margin-start: 80px;
}

#errorLongContent {
  -moz-margin-start: 80px;
}

#errorShortDesc > p {
  border-bottom: 1px solid ThreeDLightShadow;
  padding-bottom: 1em;
  font-size: 130%;
}

#errorLongDesc {
  -moz-padding-end: 3em;
  font-size: 110%;
}

#errorLongDesc > p {
}

#errorTryAgain {
  margin-top: 2em;
  -moz-margin-start: 80px;
}

#brand {
  position: absolute;
  right: 0;
  bottom: -1.5em;
  -moz-margin-end: 10px;
  opacity: .4;
}

body[dir="rtl"] #brand {
  right: auto;
  left: 0;
}

#brand > p {
  margin: 0;
}

#errorContainer {
  display: none;
}
