/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Communicator client code, released
 * March 31, 1998.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998-1999
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Joe Hewitt (hewitt@netscape.com)
 *   Kevin Gerich (webmail@kmgerich.com)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* ===== button.css =====================================================
  == Styles used by the XUL button element.
  ======================================================================= */

@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");

/* :::::::::: button :::::::::: */

button {
  -moz-appearance: button;
  margin: 6px;
  min-width: 6.3em;
  -moz-appearance: button;
 padding: 0px 4px;
  color: #000000;
}

.button-text {
  margin: 0 !important;
  text-align: center;
}

button[type="default"] {
	font: menu;
}

/* .......... active/open/checked state .......... */

button:hover:active,
button[open="true"],
button[checked="true"] {

}

/* .......... disabled state .......... */

button[disabled="true"] {
  color: #777777 !important;
  -moz-border-top-colors: #cccccc #eeeeee #fdfdfd;
  -moz-border-right-colors: #cccccc #d9d9d9;
  -moz-border-bottom-colors: #cccccc #eeeeee #fdfdfd;
  -moz-border-left-colors: #cccccc #d9d9d9;
}

/* ::::: menu/menu-button buttons ::::: */

button[type="menu-button"] {
  -moz-box-align: center;
  -moz-box-pack: center;
  margin: 0;
  border: none;
}

.button-menu-dropmarker,
.button-menubutton-dropmarker {
  margin: 1px;
  list-style-image: url("chrome://global/skin/arrow/arrow-dn.png");
}
  
.button-menu-dropmarker[disabled="true"],
.button-menubutton-dropmarker[disabled="true"] {
  list-style-image: url("chrome://global/skin/arrow/arrow-dn-dis.png");
}

.button-menubutton-dropmarker[open="true"] {
  /* margin: 2px 0px 0px 2px; */
}

/* ::::: plain buttons ::::: */

button.plain {
  margin: 0px !important;
  padding: 0px !important;
}

/* ::::: close button ::::: */

.close-button {
  list-style-image: url("chrome://global/skin/icons/close-button.gif");
}

.close-button > .toolbarbutton-icon {
  margin: 0;
}

.close-button > .toolbarbutton-text {
  display: none;
}

.dialog-button {
	font: menu;
}
